@@ -19,25 +19,28 @@ enum UIMESSAGE_CODE {
   UIERROR_SUBHEADERBROKEN, UIERROR_SUBHEADERUNKNOWN,
   UIERROR_SUBHEADERDATABROKEN, UIERROR_RRDAMAGED, UIERROR_UNKNOWNMETHOD,
   UIERROR_UNKNOWNENCMETHOD, UIERROR_RENAMING, UIERROR_NEWERRAR,
-  UIERROR_NOTSFX, UIERROR_OLDTOSFX,
-  UIERROR_WRONGSFXVER, UIERROR_HEADENCMISMATCH, UIERROR_DICTOUTMEM,
+  UIERROR_NOTSFX, UIERROR_OLDTOSFX,UIERROR_WRONGSFXVER, 
+  UIERROR_HEADENCMISMATCH, UIERROR_DICTOUTMEM,UIERROR_EXTRDICTOUTMEM,
   UIERROR_USESMALLERDICT, UIERROR_MODIFYUNKNOWN, UIERROR_MODIFYOLD,
   UIERROR_MODIFYLOCKED, UIERROR_MODIFYVOLUME, UIERROR_NOTVOLUME,
   UIERROR_NOTFIRSTVOLUME, UIERROR_RECVOLLIMIT, UIERROR_RECVOLDIFFSETS,
   UIERROR_RECVOLALLEXIST, UIERROR_RECVOLFOUND, UIERROR_RECONSTRUCTING,
   UIERROR_RECVOLCANNOTFIX, UIERROR_OPFAILED, UIERROR_UNEXPEOF,
-  UIERROR_BADARCHIVE, UIERROR_CMTBROKEN, UIERROR_INVALIDNAME,
-  UIERROR_NEWRARFORMAT, UIERROR_NOTSUPPORTED, UIERROR_ENCRNOTSUPPORTED,
-  UIERROR_RARZIPONLY, UIERROR_REPAIROLDFORMAT, UIERROR_NOFILESREPAIRED,
-  UIERROR_NOFILESTOADD, UIERROR_NOFILESTODELETE, UIERROR_NOFILESTOEXTRACT,
-  UIERROR_MISSINGVOL, UIERROR_NEEDPREVVOL, UIERROR_UNKNOWNEXTRA,
-  UIERROR_CORRUPTEXTRA, UIERROR_NTFSREQUIRED, UIERROR_ZIPVOLSFX,
-  UIERROR_FILERO, UIERROR_TOOLARGESFX, UIERROR_NOZIPSFX, UIERROR_EMAIL,
-  UIERROR_ACLGET, UIERROR_ACLBROKEN, UIERROR_ACLUNKNOWN, UIERROR_ACLSET,
-  UIERROR_STREAMBROKEN, UIERROR_STREAMUNKNOWN, UIERROR_INCOMPATSWITCH,
-  UIERROR_PATHTOOLONG, UIERROR_DIRSCAN, UIERROR_UOWNERGET,
-  UIERROR_UOWNERBROKEN, UIERROR_UOWNERGETOWNERID, UIERROR_UOWNERGETGROUPID,
-  UIERROR_UOWNERSET, UIERROR_ULINKREAD, UIERROR_ULINKEXIST,
+  UIERROR_TRUNCSERVICE, UIERROR_BADARCHIVE, UIERROR_CMTBROKEN, 
+  UIERROR_INVALIDNAME, UIERROR_NEWRARFORMAT, UIERROR_NOTSUPPORTED,
+  UIERROR_ENCRNOTSUPPORTED, UIERROR_RARZIPONLY, UIERROR_REPAIROLDFORMAT,
+  UIERROR_NOFILESREPAIRED, UIERROR_NOFILESTOADD, UIERROR_NOFILESTODELETE,
+  UIERROR_NOFILESTOEXTRACT, UIERROR_MISSINGVOL, UIERROR_NEEDPREVVOL,
+  UIERROR_UNKNOWNEXTRA, UIERROR_CORRUPTEXTRA, UIERROR_NTFSREQUIRED,
+  UIERROR_ZIPVOLSFX, UIERROR_FILERO, UIERROR_TOOLARGESFX, UIERROR_NOZIPSFX,
+  UIERROR_NEEEDSFX64, UIERROR_EMAIL, UIERROR_ACLGET, UIERROR_ACLBROKEN,
+  UIERROR_ACLUNKNOWN, UIERROR_ACLSET, UIERROR_STREAMBROKEN,
+  UIERROR_STREAMUNKNOWN, UIERROR_INCOMPATSWITCH, UIERROR_PATHTOOLONG,
+  UIERROR_DIRSCAN, UIERROR_UOWNERGET, UIERROR_UOWNERBROKEN,
+  UIERROR_UOWNERGETOWNERID, UIERROR_UOWNERGETGROUPID, UIERROR_UOWNERSET,
+  UIERROR_ULINKREAD, UIERROR_ULINKEXIST, UIERROR_OPENPRESERVEATIME,
+  UIERROR_READERRTRUNCATED, UIERROR_READERRCOUNT, UIERROR_DIRNAMEEXISTS,
+  UIERROR_TRUNCPSW, UIERROR_ADJUSTVALUE, UIERROR_SKIPUNSAFELINK,
 
   UIMSG_FIRST,
   UIMSG_STRING, UIMSG_BUILD, UIMSG_RRSEARCH, UIMSG_ANALYZEFILEDATA,
@@ -47,6 +50,7 @@ enum UIMESSAGE_CODE {
   UIMSG_CORRECTINGNAME, UIMSG_BADARCHIVE, UIMSG_CREATING, UIMSG_RENAMING,
   UIMSG_RECVOLCALCCHECKSUM, UIMSG_RECVOLFOUND, UIMSG_RECVOLMISSING,
   UIMSG_MISSINGVOL, UIMSG_RECONSTRUCTING, UIMSG_CHECKSUM, UIMSG_FAT32SIZE,
+  UIMSG_SKIPENCARC, UIMSG_FILERENAME, 
 
   UIWAIT_FIRST,
   UIWAIT_DISKFULLNEXT, UIWAIT_FCREATEERROR, UIWAIT_BADPSW,
@@ -73,34 +77,44 @@ enum UIASKREP_RESULT {
   UIASKREP_R_RENAME,UIASKREP_R_RENAMEAUTO,UIASKREP_R_CANCEL,UIASKREP_R_UNUSED
 };
 
-UIASKREP_RESULT uiAskReplace(wchar *Name,size_t MaxNameSize,int64 FileSize,RarTime *FileTime,uint Flags);
-UIASKREP_RESULT uiAskReplaceEx(RAROptions *Cmd,wchar *Name,size_t MaxNameSize,int64 FileSize,RarTime *FileTime,uint Flags);
+UIASKREP_RESULT uiAskReplace(std::wstring &Name,int64 FileSize,RarTime *FileTime,uint Flags);
+UIASKREP_RESULT uiAskReplaceEx(CommandData *Cmd,std::wstring &Name,int64 FileSize,RarTime *FileTime,uint Flags);
 
 void uiInit(SOUND_NOTIFY_MODE Sound);
 
 
-void uiStartArchiveExtract(bool Extract,const wchar *ArcName);
-bool uiStartFileExtract(const wchar *FileName,bool Extract,bool Test,bool Skip);
+void uiStartArchiveExtract(bool Extract,const std::wstring &ArcName);
+bool uiStartFileExtract(const std::wstring &FileName,bool Extract,bool Test,bool Skip);
 void uiExtractProgress(int64 CurFileSize,int64 TotalFileSize,int64 CurSize,int64 TotalSize);
 void uiProcessProgress(const char *Command,int64 CurSize,int64 TotalSize);
 
-enum UIPASSWORD_TYPE {UIPASSWORD_GLOBAL,UIPASSWORD_FILE,UIPASSWORD_ARCHIVE};
-bool uiGetPassword(UIPASSWORD_TYPE Type,const wchar *FileName,SecPassword *Password);
+enum UIPASSWORD_TYPE
+{
+  UIPASSWORD_GLOBAL,  // For -p, -hp without parameter or Ctrl+P in WinRAR.
+  UIPASSWORD_FILE,    // Extracting an encrypted file.
+  UIPASSWORD_ARCHIVE, // Extracting or opening an encrypted header archive.
+};
+
+bool uiGetPassword(UIPASSWORD_TYPE Type,const std::wstring &FileName,SecPassword *Password,CheckPassword *CheckPwd);
 bool uiIsGlobalPasswordSet();
 
 enum UIALARM_TYPE {UIALARM_ERROR, UIALARM_INFO, UIALARM_QUESTION};
 void uiAlarm(UIALARM_TYPE Type);
 
+void uiEolAfterMsg();
 
-bool uiAskNextVolume(wchar *VolName,size_t MaxSize);
-bool uiAskRepeatRead(const wchar *FileName);
-bool uiAskRepeatWrite(const wchar *FileName,bool DiskFull);
+bool uiAskNextVolume(std::wstring &VolName);
+#if !defined(SILENT) && !defined(SFX_MODULE)
+void uiAskRepeatRead(const std::wstring &FileName,bool &Ignore,bool &All,bool &Retry,bool &Quit);
+#endif
+bool uiAskRepeatWrite(const std::wstring &FileName,bool DiskFull);
+
+bool uiDictLimit(CommandData *Cmd,const std::wstring &FileName,uint64 DictSize,uint64 MaxDictSize);
 
 #ifndef SFX_MODULE
-const wchar *uiGetMonthName(int Month);
+const wchar *uiGetMonthName(uint Month);
 #endif
 
-
 class uiMsgStore
 {
   private:
@@ -126,6 +140,12 @@ class uiMsgStore
         Str[StrSize++]=s;
       return *this;
     }
+    uiMsgStore& operator << (const std::wstring &s)
+    {
+      if (StrSize<MAX_MSG)
+        Str[StrSize++]=s.c_str();
+      return *this;
+    }
     uiMsgStore& operator << (uint n)
     {
       if (NumSize<MAX_MSG)
@@ -137,33 +157,31 @@ class uiMsgStore
 };
 
 
-// Templates recognize usual NULL as integer, not wchar*.
-#define UINULL ((wchar *)NULL)
-
-inline void uiMsg(UIMESSAGE_CODE Code)
+inline void uiMsgBase(uiMsgStore &Store)
 {
-  uiMsgStore Store(Code);
-  Store.Msg();
+  // Called last, when no parameters are left.
 }
 
-template<class T1> void uiMsg(UIMESSAGE_CODE Code,T1 a1)
+template<class T1,class... TN> void uiMsgBase(uiMsgStore &Store,T1&& a1,TN&&... aN)
 {
-  uiMsgStore Store(Code);
+  // Process first parameter and pass the rest to same uiMsgBase.
   Store<<a1;
-  Store.Msg();
+  uiMsgBase(Store,aN...);
 }
 
-template<class T1,class T2> void uiMsg(UIMESSAGE_CODE Code,T1 a1,T2 a2)
+
+// Use variadic templates.
+//
+// We must pass variable parameters by reference, so no temporary copies are
+// created for custom string objects like CStringBase in 7-Zip decompression
+// code. Such temporary copies would be destroyed inside of recursive
+// uiMsgBase calls, leaving us with Str[] items pointing at released memory.
+// Since we pass integer values as well, we can't use & references
+// and must resort to && rvalue references.
+template<class... TN> void uiMsg(UIMESSAGE_CODE Code,TN&&... aN)
 {
   uiMsgStore Store(Code);
-  Store<<a1<<a2;
-  Store.Msg();
-}
-
-template<class T1,class T2,class T3> void uiMsg(UIMESSAGE_CODE code,T1 a1,T2 a2,T3 a3)
-{
-  uiMsgStore Store(code);
-  Store<<a1<<a2<<a3;
+  uiMsgBase(Store,aN...);
   Store.Msg();
 }
 
